
/*******************************************

 Fire up MS Excel, method 2 by Chris Roper.

 *******************************************/

options noxsync noxwait;
filename sas2xl dde 'excel|system';
data _null_;
  length fid rc start stop time 8;
  fid=fopen('sas2xl','s');
  if (fid le 0) then do;
    rc=system('start excel');
    start=datetime();
    stop=start+10;
    do while (fid le 0);
      fid=fopen('sas2xl','s');
      time=datetime();
      if (time ge stop) then fid=1;
      end;
    end;
  rc=fclose(fid);
run;



/*********************************

 Open an existing Excel workbook:
 'existing workbook.xls'

 *********************************/

data _null_;
  file sas2xl;
  put '[open("d:\koen\sugi 26 paper\existing workbook")]';
run;



/**************************************

 Save the workbook before modifying as
 'Saved Before Modifying.xls'

 **************************************/

data _null_;
  file sas2xl;
  put '[save.as("d:\koen\sugi 26 paper\Saved Before Modifying")]';
  put '[file.close(false)]';
run;



/************************************

 Open 'existing workbook.xls' again.

 ************************************/

data _null_;
  file sas2xl;
  put '[open("d:\koen\sugi 26 paper\existing workbook")]';
run;



/*************************************

 Create some dummy SAS data: a bit of
 text, a bit of numerics, and a date.

 *************************************/

data price_list;
  length
    band album $20
    date price 8
    ;
  informat
    date date9.
    ;
  input
    band  $1-10
    album $12-23
    @25 date
    @35 price
    ;
  cards;
Waits, Tom Bone Machine 23FEB1992 19.95
VdGG       Godbluff     15MAY1975 8.50
Pixies     Bossanova    08NOV1990 14.99
Cave, Nick Tender Prey  10AUG1988 17.49
;
run;



/*************************************

 Define a DDE triplet-style fileref
 pointing to the cell-range R1C1:R4C4
 of the 'Records' sheet.

 *************************************/

filename
  recrange
  dde
  'excel|[existing workbook.xls]records!r1c1:r4c4'
  notab;



/***************************************

 Write SAS data to the 'Records' sheet.

 ***************************************/

data _null_;
  set price_list;
  file recrange;
  put band '09'x album '09'x date '09'x price;
run;



/*********************************************

 Add a SAS date format to the dummy SAS data.

 *********************************************/

proc datasets library=work nolist;
  modify price_list;
    format date date9.;
quit;



/*****************************************

 Re-populate the 'Records' sheet with the
 same SAS data set.

 *****************************************/

data _null_;
  set price_list;
  file recrange;
  put band '09'x album '09'x date '09'x price;
run;



/********************************************

 Apply some formatting to the inserted data.

 ********************************************/

data _null_;
  file sas2xl;
  put '[error(false)]';
  put '[workbook.activate("records",false)]';
  put '[select("r1c1:r4c4")]';
  put '[format.font("Verdana",24,false,false,false,false,4,false,false)]';
  put '[patterns(1,0,6)]';
  put '[column.width(0,"c1:c4",false,3)]';
run;



/*********************************************

 Insert a new sheet into the active workbook.

 *********************************************/

data _null_;
  file sas2xl;
  put '[workbook.next()]';
  put '[workbook.insert(1)]';
  put '[workbook.move("sheet1","existing workbook.xls",1)]';
run;



/******************************************

 Attempt to rename the new sheet 'Sheet1'.

 ******************************************/

data _null_;
  file sas2xl;
  put '[workbook.name("Sheet1","No Go")]';
run;



/*****************************************

 Insert a new macro-sheet into the active
 workbook, move it to the top.

 *****************************************/

data _null_;
  file sas2xl;
  put '[workbook.next()]';
  put '[workbook.insert(3)]';
  put '[workbook.move("macro1","existing workbook.xls",1)]';
run;



/***************************************

 Define a DDE triplet-style fileref
 pointing to the cell-range R1C1:R100C1
 of the 'Macro1' sheet.

 ***************************************/

filename
  xlmacro
  dde
  "excel|macro1!r1c1:r100c1"
  notab
  lrecl=200;



/*****************************************

 Use SAS to write an Excel macro into the
 'Macro1' sheet, and run it.

 *****************************************/

data _null_;
  file xlmacro;
  put '=workbook.name("sheet1","Works Fine Now")';
  put '=halt(true)';
  put '!dde_flush';
  file sas2xl;
  put '[run("macro1!r1c1")]';
  put '[error(false)]';
run;



/**************************

 Clear the 'Macro1' sheet.

 **************************/

data _null_;
  file sas2xl;
  put '[workbook.activate("macro1")]';
  put '[select("r1c1:r100c1")]';
  put '[clear(1)]';
  put '[select("r1c1")]';
run;



/**********************************

 Define the SAS macro 'LOADNAMES'.

 **********************************/

%macro loadnames;
  %local sh wn nsheets;
  %let sh=0;
  %let wn=0;
  %let nsheets=0;
  data _null_;
    file xlmacro;
    put '=set.value($b$1,get.workbook(4))';
    put '=halt(true)';
    put '!dde_flush';
    file sas2xl;
    put '[run("macro1!r1c1")]';
    put '[error(false)]';
  run;
  filename nsheets dde "excel|macro1!r1c2:r1c2" notab lrecl=200;
  data _null_;
    length nsheets 8;
    infile nsheets;
    input nsheets;
    call symput('nsheets',trim(left(put(nsheets,2.))));
  run;
  %let nsheets=%eval(&nsheets-1);
  data _null_;
    file sas2xl;
    put '[workbook.activate("macro1")]';
    put '[select("r1c1:r100c1,r1c2:r1c2")]';
    put '[clear(1)]';
    put '[select("r1c1")]';
  run;
  data _null_;
    length maccmd $200;
    file xlmacro;
    %do sh=1 %to &nsheets;
      maccmd="=select(!$b$&sh,!$b$&sh)";
      put maccmd;
      put '=set.name("cell",selection())';
      %do wn=1 %to &sh;
        put '=workbook.next()';
        %end;
      put '=set.value(cell,get.workbook(3))';
      put '=workbook.activate("Macro1")';
      %end;
    put '=halt(true)';
    put '!dde_flush';
    file sas2xl;
    put '[run("macro1!r1c1")]';
    put '[error(false)]';
  run;
  filename sheets dde "excel|macro1!r1c2:r&nsheets.c2" lrecl=200;
  data _sheet_names;
    length bookname sheetname $100;
    infile sheets delimiter=']';
    input bookname sheetname;
    bookname=substr(bookname,2);
    bookname=left(reverse(substr(left(reverse(bookname)),5)));
  run;
  filename nsheets clear;
  filename sheets clear;
%mend loadnames;



/*******************************

 Run the 'LOADNAMES' SAS macro.

 *******************************/

%loadnames;



/**********************************

 A shorter way, but not altogether
 satisfying ...

 **********************************/

filename topics dde 'excel|system!topics';

data _null_;
  length topics $1000;
  infile topics pad dsd notab dlm='09'x;
  input topics $ @@;
  put topics;
run;




/*************************************

 Create some more numerical SAS data.

 *************************************/

data _sinuous(drop=counter angle);
  length x y z counter angle 8;
  do counter=1 to 1000;
    angle=counter/80;
    x=sin(angle);
    y=cos(angle*x);
    z=x*(y-1)+y;
    output;
    end;
run;



/********************************************

 Use the 'SASTOXL' macro to dump the SAS
 data set _SINUOUS into a 1-sheet Excel
 workbook with custom sheet- and file-names.

 ********************************************/

%sastoxl(
         libin    = work,
         dsin     = _sinuous,
         sheet    = Sinuous Numbers from SAS,
         savepath = d:\koen\sugi 26 paper,
         savename = Made by the SAS System,
         stdfmtng = 1
        );



/********************************************

 Use the 'SASTOXL' macro to dump the SAS
 data set _SINUOUS into the 'Random Numbers'
 sheet of the 'Existing Workbook.xls'.

 ********************************************/

%sastoxl(
         libin    = work,
         dsin     = _sinuous,
         tmplpath = d:\koen\sugi 26 paper,
         tmplname = existing workbook,
         sheet    = random numbers,
         savepath = d:\koen\sugi 26 paper,
         savename = Updated Workbook,
         stdfmtng = 1
        );

